from decimal import Decimal
from typing import List, Optional, Union

from .helpers import Item, get_currency


def serialize_operation(
    merchant: int,
    operation_id: str,
    currency: Union[str, int],
    amount: Decimal,
    city: str,
    country_code: str,
    state_code: str,
    billing_first_name: str,
    billing_last_name: str,
    billing_email: str,
    billing_phone: str,
    billing_postcode: str,
    billing_address_1: str,
    billing_address_2: Optional[str],
    customer_id: str,
    customer_ip_address: str,
    items: List[Item],
):
    cur = get_currency(currency)
    return {
        "MERCHANT": merchant,
        "OPERATIONID": operation_id,
        "CURRENCYCODE": cur.numeric.zfill(3),
        "AMOUNT": "%.2f" % (amount),
        "TIMEOUT": "300000",
        "CSBTCITY": city,
        "CSSTCITY": city,
        "CSBTCOUNTRY": country_code,
        "CSSTCOUNTRY": country_code,
        "CSBTEMAIL": billing_email,
        "CSSTEMAIL": billing_email,
        "CSBTFIRSTNAME": billing_first_name,
        "CSSTFIRSTNAME": billing_first_name,
        "CSBTLASTNAME": billing_last_name,
        "CSSTLASTNAME": billing_last_name,
        "CSBTPHONENUMBER": billing_phone,
        "CSSTPHONENUMBER": billing_phone,
        "CSBTPOSTALCODE": billing_postcode,
        "CSSTPOSTALCODE": billing_postcode,
        "CSBTSTATE": state_code,
        "CSSTSTATE": state_code,
        "CSBTSTREET1": billing_address_1,
        "CSSTSTREET1": billing_address_1,
        "CSBTSTREET2": billing_address_2,
        "CSSTSTREET2": billing_address_2,
        "CSBTCUSTOMERID": str(customer_id),
        "CSBTIPADDRESS": customer_ip_address,
        "CSPTCURRENCY": cur.alpha,
        "CSPTGRANDTOTALAMOUNT": "%.2f" % (amount),
        "CSITPRODUCTCODE": str("default#" * len(items))[:-1],
        "CSITPRODUCTDESCRIPTION": "#".join([i.description for i in items]),
        "CSITPRODUCTNAME": "#".join([i.name for i in items]),
        "CSITPRODUCTSKU": "#".join([i.sku for i in items]),
        "CSITTOTALAMOUNT": "#".join(["%.2f" % (i.amount) for i in items]),
        "CSITQUANTITY": "#".join([str(i.quantity) for i in items]),
        "CSITUNITPRICE": "#".join(["%.2f" % (i.unit_price) for i in items]),
    }


def serialize_merchant(token: str, merchant: int, success_url: str, failure_url: str):
    return {
        "Security": token[-32:],
        "Merchant": merchant,
        "URL_OK": success_url,
        "URL_ERROR": failure_url,
    }


def serialize_gaa(token: str, merchant: int, request_key: str, answer_key: str):
    return {
        "Security": token[-32:],
        "Merchant": merchant,
        "RequestKey": request_key,
        "AnswerKey": answer_key,
    }



from zeep import Client, Settings
from zeep.cache import SqliteCache
from zeep.transports import Transport

wsdl = "python_todopago/wsdl/Authorize.wsdl"
ENDPOINTS = {
    True: "https://developers.todopago.com.ar/services/t/1.1/",
    False: "https://apis.todopago.com.ar/services/t/1.1/",
}


def get_client(token: str, sandbox: bool = False) -> Client:
    endpoiont = ENDPOINTS[sandbox]
    settings = Settings(extra_http_headers={"Authorization": token})
    transport = Transport(cache=SqliteCache(timeout=86400))
    client = Client(
        endpoiont + "Authorize?wsdl",
        settings=settings,
        transport=transport,
    )
    client.service._binding_options["address"] = endpoiont + "Authorize"
    return client



import pytest

from python_todopago import helpers


def get_fieldname():
    string = helpers.get_fieldname("ARS")
    numeric_a = helpers.get_fieldname("32")
    numeric_b = helpers.get_fieldname(32)

    assert string == "alphabetic_code"
    assert numeric_a == "numeric_code"
    assert numeric_b == "numeric_code"


def test_get_currency_by_alphabetic_code():
    currency = helpers.get_currency("ARS")
    assert currency.alpha == "ARS"


def test_get_currency_by_numeric_code():
    currency_a = helpers.get_currency(32)
    currency_b = helpers.get_currency("32")
    assert currency_a.alpha == currency_b.alpha == "ARS"


def test_get_currency_with_invalidcode():
    # Test with invalid code
    with pytest.raises(ValueError):
        _ = helpers.get_currency("AAA")

    # Test with invalid code type
    with pytest.raises(TypeError):
        _ = helpers.get_currency(32.5)



from decimal import Decimal

import pytest

from python_todopago.helpers import Item


@pytest.fixture
def items():
    return [
        Item("Harry Potter", "A book", "1234", Decimal(1.0), 1, Decimal(1.0)),
        Item("Harry Potter", "A book", "1234", Decimal(1.0), 1, Decimal(1.0)),
    ]


@pytest.fixture
def operation_info(items):
    return {
        "success_url": "http://example.com/success/",
        "failure_url": "http://example.com/failure/",
        "operation_id": "ABC",
        "currency": 32,
        "amount": 2.00,
        "city": "Cordoba",
        "country_code": "AR",
        "state_code": "D",
        "billing_first_name": "Juan",
        "billing_last_name": "Lopez",
        "billing_email": "test@gmail.com",
        "billing_phone": "+543513840243",
        "billing_postcode": "5000",
        "billing_address_1": "Arrayan 8958",
        "billing_address_2": None,
        "customer_id": "1",
        "customer_ip_address": "192.168.0.1",
        "items": items,
    }



import json
import logging
from dataclasses import asdict
from decimal import Decimal
from typing import List, Optional, Union

import requests

from .clients import Client, get_client
from .exceptions import InvalidCredentialsException
from .helpers import Authorization, Credentials, Item, OperationStatus, object_to_xml
from .serializers import serialize_gaa, serialize_merchant, serialize_operation

logging.getLogger("zeep").setLevel(logging.ERROR)
API = "https://apis.todopago.com.ar/api"


class TodoPagoConnector:
    success_url: str
    failure_url: str
    token: str
    merchant: int
    client: Client
    sandbox: bool

    def __init__(
        self,
        token: str = None,
        merchant: int = None,
        username: str = None,
        password: str = None,
        sandbox: bool = False,
    ):
        self.sandbox = sandbox

        if not token and (username and password):
            credentials = TodoPagoConnector.get_credentials(username, password)
            if None in asdict(credentials).values():
                raise InvalidCredentialsException()

        self.token = token or credentials.token
        self.merchant = merchant or credentials.merchant
        self.client = get_client(self.token, self.sandbox)

    @staticmethod
    def get_credentials(username: str, password: str) -> Credentials:
        body = json.dumps({"USUARIO": username, "CLAVE": password})
        res = requests.post(
            API + "/Credentials",
            body,
            headers={"Content-Type": "application/json"},
        )
        data = res.json().get("Credentials")
        return Credentials(
            merchant=data.get("merchantId", None), token=data.get("APIKey", None)
        )

    def authorize_operation(
        self,
        success_url: str,
        failure_url: str,
        operation_id: str,
        currency: Union[str, int],
        amount: Decimal,
        city: str,
        country_code: str,
        state_code: str,
        billing_first_name: str,
        billing_last_name: str,
        billing_email: str,
        billing_phone: str,
        billing_postcode: str,
        billing_address_1: str,
        billing_address_2: Optional[str],
        customer_id: str,
        customer_ip_address: str,
        items: List[Item],
    ) -> Authorization:
        req_body = serialize_merchant(
            self.token, self.merchant, success_url, failure_url
        )
        operation = serialize_operation(
            self.merchant,
            operation_id,
            currency,
            amount,
            city,
            country_code,
            state_code,
            billing_first_name,
            billing_last_name,
            billing_email,
            billing_phone,
            billing_postcode,
            billing_address_1,
            billing_address_2,
            customer_id,
            customer_ip_address,
            items,
        )
        req_body.update({"Payload": object_to_xml(operation, "Request")})
        res = self.client.service.SendAuthorizeRequest(**req_body)
        return Authorization(
            res.StatusCode,
            res.StatusMessage,
            res.URL_Request,
            res.RequestKey,
            res.PublicRequestKey,
        )

    def get_operation_status(
        self, request_key: str, answer_key: str
    ) -> OperationStatus:
        req_body = serialize_gaa(self.token, self.merchant, request_key, answer_key)
        res = self.client.service.GetAuthorizeAnswer(**req_body)
        return OperationStatus(res.StatusCode, res.StatusMessage, res.AuthorizationKey)



import json
from dataclasses import dataclass
from decimal import Decimal
from os import path
from typing import Optional, Union


@dataclass(frozen=True)
class Item:
    """A dataclass holding an operation item information"""

    description: str
    name: str
    sku: str
    amount: Decimal
    quantity: int
    unit_price: Decimal


@dataclass(frozen=True)
class Authorization:
    """Response details from SendAuthorizeRequest service"""

    status_code: int
    status_message: str
    form_url: Optional[str]
    request_key: Optional[str]
    public_request_key: Optional[str]


@dataclass(frozen=True)
class OperationStatus:
    """Response details from GetAuthorizedAnswer service"""

    status_code: int
    status_message: str
    authorization_key: str


@dataclass(frozen=True)
class Credentials:
    """Response details from /api/Credentials"""

    merchant: int
    token: str


@dataclass(frozen=True)
class Currency:
    """ISO 4217 currency code"""

    alpha: str
    numeric: str


def get_currency(code: Union[int, str]) -> Currency:
    """
    Get currency code by numeric or alphabetic code,
    if there is no match raises ValueError.
    These codes are based on ISO 4217 standard.

    :param code: numeric or alphabetic code
    """
    field = get_fieldname(code)

    basedir = path.dirname(__file__)
    with open(basedir + "/iso4217.json") as data:
        currencies = json.load(data)
        currency = next(
            (
                Currency(c["alphabetic_code"], str(c["numeric_code"]))
                for c in currencies
                if str(c[field]) == str(code)
            ),
            None,
        )

    if not currency:
        raise ValueError(f"Invalid currency code: {code}")
    return currency


def get_fieldname(code: Union[int, str]) -> str:
    """
    Get the field name based on the field name.
    """
    if isinstance(code, int):
        return "numeric_code"
    elif isinstance(code, str):
        return "alphabetic_code" if code.isalpha() else "numeric_code"
    raise TypeError(f"Invalid type for code: {type(code)}, expected int or str.")


def object_to_xml(data: Union[dict, bool], root="object"):
    xml = f"<{root}>"
    if isinstance(data, dict):
        for key, value in data.items():
            xml += object_to_xml(value, key)

    elif isinstance(data, (list, tuple, set)):
        for item in data:
            xml += object_to_xml(item, "item")

    else:
        xml += str(data)

    xml += f"</{root}>"
    return xml



#!/usr/bin/env python3
from pathlib import Path

from setuptools import find_packages, setup

this_directory = Path(__file__).parent
long_description = (this_directory / "README.md").read_text()

setup(
    version="1.0.6",
    name="python-todopago",
    description="TodoPago modern SDK for python",
    long_description=long_description,
    long_description_content_type="text/markdown",
    author="Juan Pablo Senn",
    author_email="juanpsenn@gmail.com",
    url="https://github.com/juanpsenn/python-todopago",
    project_urls={
        "Issue Tracker": "https://github.com/juanpsenn/python-todopago/issues",
    },
    license="MIT",
    packages=find_packages(),
    data_files=[("python_todopago", ["python_todopago/iso4217.json"])],
    include_package_data=True,
    install_requires=[
        "requests>=2.25.0",
        "zeep>=4.0.0,<5.0.0",
    ],
    extra_requires={
        "testing": [
            "pytest",
            "pytest-cov",
            "requests-mock",
            "tox",
        ]
    },
    classifiers=[
        "Intended Audience :: Developers",
        "License :: OSI Approved :: MIT License",
        "Operating System :: OS Independent",
        "Programming Language :: Python :: 3",
        "Programming Language :: Python :: 3.7",
        "Programming Language :: Python :: 3.8",
        "Programming Language :: Python :: 3.9",
        "Topic :: Software Development :: Libraries :: Python Modules",
    ],
)



import pytest
import requests_mock

from python_todopago import TodoPagoConnector
from python_todopago.clients import ENDPOINTS


@pytest.fixture(scope="module")
def connector():
    return TodoPagoConnector(
        "PRISMA A793D307441615AF6AAAD7497A75DE59",
        2159,
        sandbox=False,
    )


def test_authorize_operation(operation_info, connector):
    authorization_response = """
    <?xml version="1.0" encoding="UTF-8"?>
    <soapenv:Envelope xmlns:soapenv="http://schemas.xmlsoap.org/soap/envelope/">
    <soapenv:Body>
        <api:SendAuthorizeRequestResponse xmlns:api="http://api.todopago.com.ar">
            <api:StatusCode>-1</api:StatusCode>
            <api:StatusMessage>Solicitud de Autorizacion Registrada</api:StatusMessage>
            <api:URL_Request>https://forms.todopago.com.ar/formulario/commands?command=formulario&amp;m=a6104bad3-1be7-4e8e-932e-e927100b2e86&amp;fr=1</api:URL_Request>
            <api:RequestKey>f5ad41bc-92ba-40ff-889d-8a23fe562a28</api:RequestKey>
            <api:PublicRequestKey>a6104bad3-1be7-4e8e-932e-e927100b2e86</api:PublicRequestKey>
        </api:SendAuthorizeRequestResponse>
    </soapenv:Body>
    </soapenv:Envelope>
    """.strip()

    with requests_mock.mock() as m:
        m.post(
            ENDPOINTS[False] + "Authorize",
            text=authorization_response,
        )
        authorization = connector.authorize_operation(**operation_info)
        assert authorization.status_code == -1


def test_get_operation_status(connector):
    status_response = """
    <?xml version="1.0" encoding="UTF-8"?>
    <soapenv:Envelope xmlns:soapenv="http://schemas.xmlsoap.org/soap/envelope/">
    <soapenv:Body>
        <api:GetAuthorizeAnswerResponse xmlns:api="http://api.todopago.com.ar">
            <api:StatusCode>-1</api:StatusCode>
            <api:StatusMessage>APROBADA</api:StatusMessage>
            <api:AuthorizationKey>817824df-8614-4ce8-a6c9-abdf884024ab</api:AuthorizationKey>
            <api:EncodingMethod>XML</api:EncodingMethod>
            <api:Payload>
                <Answer xmlns="http://api.todopago.com.ar">
                <DATETIME>2022-01-01T20:46:03 -0300</DATETIME>
                <CURRENCYNAME>Peso Argentino</CURRENCYNAME>
                <PAYMENTMETHODNAME>VISA DEBITO</PAYMENTMETHODNAME>
                <TICKETNUMBER>5770</TICKETNUMBER>
                <AUTHORIZATIONCODE>646962</AUTHORIZATIONCODE>
                <CARDNUMBERVISIBLE>4517XXXXXXXX6388</CARDNUMBERVISIBLE>
                <BARCODE />
                <OPERATIONID>ABC</OPERATIONID>
                <COUPONEXPDATE />
                <COUPONSECEXPDATE />
                <COUPONSUBSCRIBER />
                <BARCODETYPE />
                <ASSOCIATEDDOCUMENTATION />
                <INSTALLMENTPAYMENTS>1</INSTALLMENTPAYMENTS>
                <OPERATIONNUMBER>56827509</OPERATIONNUMBER>
                <CFT>0</CFT>
                <TEA>0</TEA>
                </Answer>
                <Request xmlns="http://api.todopago.com.ar">
                <MERCHANT>2033247</MERCHANT>
                <OPERATIONID>ABC</OPERATIONID>
                <AMOUNT>2.00</AMOUNT>
                <CURRENCYCODE>32</CURRENCYCODE>
                <AMOUNTBUYER>2.00</AMOUNTBUYER>
                <BANKID>18</BANKID>
                <PROMOTIONID />
                </Request>
            </api:Payload>
        </api:GetAuthorizeAnswerResponse>
    </soapenv:Body>
    </soapenv:Envelope>
    """.strip()

    with requests_mock.mock() as m:
        m.post(
            ENDPOINTS[False] + "Authorize",
            text=status_response,
        )
        status = connector.get_operation_status(
            "1fb7cc9a-14dd-42ec-bf1e-6d5820799642",
            "44caba31-1373-4544-aa6b-42abff696944",
        )
        assert status.status_code == -1
        assert status.status_message == "APROBADA"
        assert status.authorization_key == "817824df-8614-4ce8-a6c9-abdf884024ab"
